$! This script is a web interface to the DCL procedure
$! SYS$MANAGER:AUTO_REPLY.COM via the web page auto_reply.html. When submited
$! as an action to a POST request by auto_reply.html, it initially runs in the
$! context of the web server to launch itself again as a remote command shell
$! using RSH running under the username password submitted by the user (user
$! context). It then calls SYS$MANAGER:AUTO_REPLY.COM  to setup an automated
$! vacation reply for the user.
$!
$! External requirements: 
$!	SET_DCL_ENV.EXE		(from WWWINDEX kit from http://www.cerritos.edu/cerritos/kits/)
$!	PMDF_EXE:UNIQUE_ID.EXE	(generates a unique ID in a DCL symbol)
$!	SITE_PUBLIC	Logical name (a place were anyone may create and delete a scratch file)
$!
$! Assumptions:
$!	HTTP_SERVER is the username of the web server
$!
$! Copyright 2000, 	C. A. Karcher
$! 			Waisman Center, University of Wisconsin-Madison
$!
$! Author: Carl Karcher, karcher@waisman.wisc.edu	Date:   22-MAR-00
$!
$! Modified:
$!
$! If we're not running under the web server, dispatch to user context
$
$ If F$Edit(F$GetJPI("","USERNAME"),"TRIM") .nes. "HTTP_SERVER" Then Goto User_Context
$
$! We are running under the web server. Setup some symbols for later use:
$
$ This_File = F$Env("PROCEDURE")
$ crlf = f$fao("!/")
$ put = "write net_link"
$ RSH$_FAILED = %X1801801A							! Status for a fail RSH authorization
$
$!------------------------------------------------------------------------------
$!
$! If it's a HEAD request, just send the Content-type and exit.
$!
$ if P1 .eqs. "HEAD"
$ then
$   put "<DNETCGI>"
$   put "Content-type: text/html", crlf, crlf	 ! CGI header
$   put "</DNETCGI>"
$   exit
$ endif
$!
$!----------------------------------------------------------------------------
$!
$ put "<DNETRECMODE>"								! Set implied carriage control.
$ mcr 'f$parse("SET_DCL_ENV.EXE;",f$environment("PROCEDURE")) "WWW_"		! Set WWW_ DCL symbols and handle long lines
$ put "Content-type: text/html", crlf						! send back a CGI header
$ put "<HTML><HEAD>"
$ put "<TITLE>Auto_Reply processing...</TITLE>", crlf
$ put "<BODY bgcolor=#ccccff>"
$ put "<H1>Auto_Reply status:</H1>"
$ put "<PRE>"									! Disable formatting
$ 
$! Cannot output a raw LF after the following command:
$
$ def sys$output net_link							! Show any errors to the user
$
$!----------------------------------------------------------------------------
$! The following loop parses field names and values from the DCL symbols set
$! by the SET_DCL_ENV utility. Symbols are named WWW_KEY_n. When n is odd the
$! symbols' value is the field name. When n is even the symbols value is the
$! field's value  WWW_KEY_COUNT contains the maximum value of n. If
$! WWW_KEY_n_COUNT exists, then the field value is multi-line of the form
$! WWW_KEY_n_l where l is the line number and n is always even. When
$! WWW_KEY_n_l  is null that means line l-1 was terminated by a CR or LF.
$! See ECHODT.COM and SET_DCL_ENV.C for more info.
$
$ Run PMDF_EXE:UNIQUE_ID							! Generate a unique ID in symbol UNIQUE_ID
$ Field_Name = "*UNDEFINED*"
$ Field_Value = "*UNDEFINED*"
$ i = 1										! Init key counter
$
$Key_loop:
$ If f$type(www_key_'i') .eqs. "STRING" Then Field_Name = www_key_'i'		! If a field name exists, save it
$ i = i + 1									! Bump key counter by one (it's now even)
$ If f$type(www_key_'i') .nes. "" 						! Does single line field value exist? 
$  Then Field_Value = www_key_'i'						!  Yes, grab it
$  'Field_name' = Field_Value							!  Assign the value to  symbol 'field_name' 
$ EndIf
$ If f$type(www_key_'i'_count) .eqs. "INTEGER"					! Multiline value?
$  Then Field_File = F$trn("site_public") + "''field_name'_''unique_id'.DAT"	!  Yes, Contruct a tread safe unique file name
$  Open/Write field_file 'Field_File' 						!  open a the file for writing lines
$  l = 1									!  Init line counter
$next_line:
$  If www_key_'i'_'l' .nes. "" Then write field_file www_key_'i'_'l' 		!  If a line exists, write it to the file
$  l = l + 1									!  bump line counter
$  if l .lt. www_key_'i'_count then goto next_line				!  Are there are more lines to do? Yes, go do them
$  close/nolog field_file							!  No, close the file
$  File 'Field_File'/prot=w:RWED						!  Allow full access to the file (so user can delete)
$ 'Field_name' = "@" + Field_File 						!  Assign  @filename to symbol 'field_name'
$ Endif
$ i = i + 1									! Bump key counter
$ if i .lt. www_key_count then goto key_loop					! Are there more form keys to do? Yes, go do them
$
$!----------------------------------------------------------------------------
$
$! We have processed all the fields from the form. MSG may be a multi-line value
$! contained in a data file. Now call ourselves back using the user/pwd supplied
$
$ Put ""
$ set noon									! Dangerous in scripts but we need to catch and error
$ rsh wcs-cluster/user="''Username'"/Pass="''Password'" -			! Create a process using supplied user/pwd
  "@''This_File' ""''Submit_Action'"" ''Username' ""''Start_Time'"" ""''Stop_Time'"" ""''Msg'"""	! and execute this file again in user context
$ If $Status .eq. RSH$_FAILED Then Put "<H2>The username and/or password you supplied were not correct</H2>"
$ Put "</PRE>"									! Close the pre-formated text block
$ put "</body></html>"								! Close document
$ exit
$
$!==============================================================================
$
$! The following part of this procedure is executed under the username and
$! password submited by the user. Here's where sys$manager:auto_reply.com
$! is called to actually setup the auto reply for the desired dates.
$!  P1 - Submit Action (Cancel, Setup or Show)
$!  P2 - Username
$!  P3 - Absolute time to start displaying the message
$!  P4 - Absolute time to stop displaying the message
$!  P5 - Message text. If the first character is "@" then the following
$!	characters refer to the file name that contains the text.
$
$User_Context:
$ On warning then goto Error_General
$ If P1 .nes. P1 - "Setup" Then GoSub SETUP
$ If P1 .nes. P1 - "Show" Then GoSub SHOW
$ If P1 .nes. P1 - "Cancel" Then GoSub CANCEL
$ Exit
$
$Show:
$ Write sys$output "<b>Auto Reply messages for user ''P2':</b>"
$ @sys$manager:auto_reply show 'P2'
$ Return
$
$Cancel:
$ @sys$manager:auto_reply disable 'P2'
$ @sys$manager:auto_reply cancel 'P2'
$ Return
$
$Setup:
$ write sys$output "<hr noshade>"
$ If P5 .nes. P5 - "@" 								! Does P5 specify an indirect file?
$  Then P5_Global = F$Extr(1,512,P5)						!  Yes, Save the global location of the message file
$  copy 'P5_Global' 'F$parse("SYS$SCRATCH",P5_Global)' 				!  Make a local copy of the message file
$  P5 = "@" + F$parse("SYS$SCRATCH",P5_Global) 					!  Make P5 point at our local copy of message file
$ EndIf
$ If P3 .nes. "" .and. P3 .eqs. P3 - " " - "+" - ":" then P3 = P3 + "+17:00"	! If P3 doesn't specify a time default to 5PM.
$ If P4 .nes. "" .and. P4 .eqs. P4 - " " - "+" - ":" then P4 = P4 + "+17:00"	! If P4 doesn't specify a time default to 5PM.
$ If F$CVT(P3) .ges. F$CVT(P4) Then Goto Error_Start_After_Stop			! Check that start time is before stop time
$ @sys$manager:auto_reply enable 'P2' "''P3'" SYS$BATCH "''P5'"
$ @sys$manager:auto_reply disable 'P2' "''P4'"
$ write sys$output ""
$ write sys$output "The following message will begin responding to e-mail<br>sent to you after ''F$CVT(P3,"ABSOLUTE")':"
$ write sys$output "<hr><em><b>"
$ If F$Search(P5-"@") .nes. ""							! Does P5 reference an existing file?
$  Then If F$Search(P5-"@") .nes. "" Then Type 'F$extr(1,512,P5)'		!  Yes, Display the local message file to the user
$  Delete 'P5_Global';								!  Delete the temporary global message file 
$ Else Write sys$output P5							!  No, display the contents of P5
$ Endif
$ write sys$output "</em><hr></b>"
$ write sys$output "The above message will stop responding to e-mail<br>sent to you after ''F$CVT(P4,"ABSOLUTE")':"
$Return
$
$Error_Start_After_Stop:
$ Write sys$output "<em>You have entered a stop time that is BEFORE the start time."
$ Write sys$output "Please click the BACK button on your browser and adjust the times.</em>"
$ Exit

$Error_General:
$ write sys$output "<hr noshade><em><b>"
$ Write sys$output "There was a problem processing your request.<br>"
$ Write sys$output "Click the BACK button on your browser and check that the times"
$ Write sys$output "you specified are in the required format. If the times are correct"
$ Write sys$output "then contact the help desk at 263-3117 and print this page for diagnosis.</em></b>"
$ Exit
